/////////////////////////////////////////////////////////////////////////
////                          SX_AD12.C                              ////
////                                                                 ////
////  This program will read both A/D channels and display the       ////
////  results as both a voltage and raw hex number over the RS-232.  ////
////  A reading is taken every second.                               ////
////                                                                 ////
////  Pin layout for the LTC1298:                                    ////
////                                                                 ////
////  SX Chip     LTC1298                                            ////
////  -------------------                                            ////
////  PIN_B0  --->  PIN 7                                            ////
////  PIN_B1  --->  PIN 6                                            ////
////  PIN_B2  --->  PIN 5                                            ////
////  PIN_B3  --->  PIN 1                                            ////
////  GND     --->  PIN 4                                            ////
////  +5      --->  PIN 8                                            ////
////                                                                 ////
////  Analog CH0 and CH1 are on pins 2 and 3 respectivly of the      ////
////  LTC1298                                                        ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,1997 Custom Computer Services            ////
//// This source code may only be used by licensed users of the CCS C   ////
//// compiler.  This source code may only be distributed to other       ////
//// licensed users of the CCS C compiler.  No other use, reproduction  ////
//// or distribution is permitted without written permission.           ////
//// Derivative programs created using this software in object code     ////
//// form are not restricted in any way.                                ////
////////////////////////////////////////////////////////////////////////////

#include <SX52BD.H>
#include "atd_emu.h"

#fuses HS,NOWDT,NOPROTECT

#use delay(clock=16000000)
#use rs232(baud=19200, xmit=PIN_A3, rcv=PIN_A2)

#include <LTC1298.C>

void display_data( long int data ) {
     char volt_string[6];

     convert_to_volts( data, volt_string );
     printf(volt_string);
     printf(" (%4lX)",data);
}

main() {
   long int value;

   set_mode_a(mTTLIN,mTTLIN,mTTLIN,mTTLIN,mOUTPUT,mTTLIN,mTTLIN,mTTLIN);
   set_mode_b(mOUTPUT,mOUTPUT,mOUTPUT,mOUTPUT,mOUTPUT,mOUTPUT,mTTLIN,mOUTPUT);

   adc_init();

   printf("\r\nSampling:\r\n");

   do {
      delay_ms(1000);

      value = read_analog(0);
      printf("\n\rCh0: ");
      display_data( value );

      value = read_analog(1);
      printf("   Ch1: ");
      display_data( value );

   } while (TRUE);
}
