/////////////////////////////////////////////////////////////////////////
////                          SX_EXTEE.C                             ////
////                                                                 ////
////  This program uses the 24xx or 93xx external EEPROM drivers to  ////
////  read and write to an external serial EEPROM.                   ////
////                                                                 ////
////  If being run from the SX 28-52 Demo board no other connections ////
////  need to be made, as a 2416 is included on the board.  Ensure   ////
////  that you include 2416.C as your external eeprom driver, and    ////
////  that J9 and J10 jumpers are shorted.                           ////
////                                                                 ////
////  Change the #include <9356.C> to any of the other drivers to    ////
////  test other parts.  Note each driver defines EEPROM_ADDRESS     ////
////  indicate 8 or 16 bit addresses.                                ////
////                                                                 ////
////  Pin layout for the 2416:                                       ////
////                                                                 ////
////  SX Chip        2416                                            ////
////  -------------------                                            ////
////  PIN_B7  --->  PIN 5                                            ////
////  PIN_B6  --->  PIN 6                                            ////
////  GND     --->  PIN 1,2,3,4,7                                    ////
////  +5      --->  PIN 8                                            ////
////                                                                 ////
////  Pin 5 and 6 on the 2416   should also be connected to +5 by    ////
////  way of a 1K resistor.                                          ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,1997 Custom Computer Services            ////
//// This source code may only be used by licensed users of the CCS C   ////
//// compiler.  This source code may only be distributed to other       ////
//// licensed users of the CCS C compiler.  No other use, reproduction  ////
//// or distribution is permitted without written permission.           ////
//// Derivative programs created using this software in object code     ////
//// form are not restricted in any way.                                ////
////////////////////////////////////////////////////////////////////////////

#include <SX52BD.H>
#include "atd_emu.h"

#fuses HS,NOWDT,NOPROTECT

#use standard_io(A)
#use delay(clock=16000000)
#use rs232(baud=19200, xmit=PIN_A3, rcv=PIN_A2)

#include <INPUT.C>
#include <2416.C>

main() {
   byte value,cmd;
   EEPROM_ADDRESS address;

   do {
      do {
         printf("\r\nRead or Write: ");
         cmd=getc();
         cmd=toupper(cmd);
         putc(cmd);
      } while ( (cmd!='R') && (cmd!='W') );

      printf("\n\rLocation: ");

#if sizeof(EEPROM_ADDRESS)==1
      address = gethex();
#else
#if EEPROM_SIZE>0xfff
      address = gethex();
#else
      address = gethex1();
#endif
      address = (address<<8)+gethex();
#endif

      if(cmd=='R')
         printf("\r\nValue: %X\r\n",READ_EXT_EEPROM( address ) );

      if(cmd=='W') {
         printf("\r\nNew value: ");
         value = gethex();
         printf("\n\r");
         WRITE_EXT_EEPROM( address, value );
      }

   } while (TRUE);

}
