/////////////////////////////////////////////////////////////////////////
////                           EX_RTCLK.C                            ////
////                                                                 ////
////  This program reads and writes to an external Real Time Clock.  ////
////  It communicates to the user using the LCD and keypad.          ////
////  Pin layout for the NJU6355:                                    ////
////                                                                 ////
////                                                                 ////
////  SX Chip     NJU6355                                            ////
////  -------------------                                            ////
////  PIN_C0  --->  PIN 7                                            ////
////  PIN_C1  --->  PIN 6                                            ////
////  PIN_C2  --->  PIN 5                                            ////
////  PIN_C3  --->  PIN 1                                            ////
////  GND     --->  PIN 4                                            ////
////  +5      --->  PIN 8                                            ////
////                                                                 ////
////  A 32Khz clock should be attached to pins 2 and 3 on the        ////
////  NJU6355 device, also.                                          ////
////                                                                 ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,1997 Custom Computer Services            ////
//// This source code may only be used by licensed users of the CCS C   ////
//// compiler.  This source code may only be distributed to other       ////
//// licensed users of the CCS C compiler.  No other use, reproduction  ////
//// or distribution is permitted without written permission.           ////
//// Derivative programs created using this software in object code     ////
//// form are not restricted in any way.                                ////
////////////////////////////////////////////////////////////////////////////

#include <SX28AC.H>
#include "atd_emu.h"

#fuses HS,NOWDT,NOPROTECT

#use standard_io(B)
#use delay(clock=16000000)
#use rs232(baud=19200, xmit=PIN_A3, rcv=PIN_A2)

#define RTC_IO    PIN_C3
#define RTC_CE    PIN_C2
#define RTC_CLK   PIN_C1
#define RTC_DATA  PIN_C0

#use standard_io(C)

#include <NJU6355.C>

#include <LCD.C>
#include <KBD.C>

byte get_bcd() {
  char first,second;

  do {
    first=kbd_getc();
  } while ((first<'0') || (first>'9'));
  lcd_putc(first);
  first-='0';

  do {
    second=kbd_getc();
  } while ((second<'0') || (second>'9'));
  lcd_putc(second);

  return((first<<4)|(second-'0'));
}

void set_clock(){
   byte day,mth,year,dow,hour,min;

   lcd_putc("\fYear 19: ");
   year=get_bcd();
   lcd_putc("\fMonth: ");
   mth=get_bcd();
   lcd_putc("\fDay: ");
   day=get_bcd();
   lcd_putc("\fWeekday 1-7: ");
   dow=get_bcd();
   lcd_putc("\fHour: ");
   hour=get_bcd();
   lcd_putc("\fMin: ");
   min=get_bcd();

   rtc_set_datetime(day,mth,year,dow,hour,min);
}


main() {
   char cmd;
   byte day,mth,year,dow,hour,min,sec;

   set_mode_a(mTTLIN,mTTLIN,mTTLIN,mTTLIN,mOUTPUT,mTTLIN,mTTLIN,mTTLIN);
   set_mode_b(mTTLIN,mTTLIN,mTTLIN,mTTLIN,mOUTPUT,mOUTPUT,mOUTPUT,mOUTPUT);

   rtc_init();
   lcd_init();
   kbd_init();

   lcd_putc("\f1: Change, 2: Display");

   do {
      cmd=kbd_getc();
   } while ((cmd!='1')&&(cmd!='2'));

   if(cmd=='1')
     set_clock();

   while (TRUE) {
      lcd_putc('\f');
      rtc_get_date( day, mth, year, dow);
      rtc_get_time( hour, min, sec );
      printf(lcd_putc,"%2X/%2X/%2X\n%2X:%2X:%2X",day,mth,year,hour,min,sec);
      delay_ms(250);
   }
}
