////////////////////////////////////////////////////////////////////////////
////                          EX_SISR.C                                 ////
////                                                                    ////
////  This program shows how to implement a interrupt service           ////
////  routine to buffer up incomming serial data.                       ////
////                                                                    ////
////  Connect pins A2 and B0 on the Scenix Demo Board for it to work.   ////
////                                                                    ////
////////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,1997 Custom Computer Services            ////
//// This source code may only be used by licensed users of the CCS C   ////
//// compiler.  This source code may only be distributed to other       ////
//// licensed users of the CCS C compiler.  No other use, reproduction  ////
//// or distribution is permitted without written permission.           ////
//// Derivative programs created using this software in object code     ////
//// form are not restricted in any way.                                ////
////////////////////////////////////////////////////////////////////////////

#include <SX52BD.H>
#include "atd_emu.h"

#fuses HS,NOWDT,NOPROTECT

#use delay(clock=16000000)
#use rs232(baud=19200, xmit=PIN_A3, rcv=PIN_A2)

//#reserve 0x20:0x2F
#define BUFFER_SIZE 16
byte buffer[BUFFER_SIZE];
//#byte buffer = 0x20

byte next_in = 0;
byte next_out = 0;

#int_rb0
void serial_isr() {
   int t;

   buffer[next_in]=getc();

   t=next_in;
   next_in=(next_in+1) % BUFFER_SIZE;
   if(next_in==next_out)
     next_in=t;           // Buffer full !!
}

#define bkbhit (next_in!=next_out)

byte bgetc() {
   byte c;

   while(!bkbhit) ;
   c=buffer[next_out];

   next_out=(next_out+1) % BUFFER_SIZE;

   return(c);
}

main() {
   set_mode_a(mTTLIN,mTTLIN,mTTLIN,mTTLIN,mOUTPUT,mTTLIN,mTTLIN,mTTLIN);
   set_mode_b(mOUTPUT,mTTLIN,mTTLIN,mTTLIN,mTTLIN,mTTLIN,mTTLIN,mTTLIN);

   setup_miwu (miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuOFF, miwuH2L);

   // The program will delay for 5 seconds and then display
   // any data that came in during the 5 second delay

   do {
      printf("\r\n\Running...\r\n");
      delay_ms(5000);

      printf("Buffered data => ");

      while(bkbhit)
         putc( bgetc() );

      printf("\r\nIn=%d Out=%d\r\n", next_in, next_out);

   } while (TRUE);
}
